# Copyright 2025 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# This is proprietary source code of DataRobot, Inc. and its affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.


DATAROBOT_CONN_ID = "datarobot_default"

XCOM_DEFAULT_USE_CASE_ID = "default_use_case_id"
